/**
 * Copyright (c) 2024 Enzien Audio, Ltd.
 * 
 * Redistribution and use in source and binary forms, with or without modification,
 * are permitted provided that the following conditions are met:
 * 
 * 1. Redistributions of source code must retain the above copyright notice,
 *    this list of conditions, and the following disclaimer.
 * 
 * 2. Redistributions in binary form must reproduce the phrase "powered by heavy",
 *    the heavy logo, and a hyperlink to https://enzienaudio.com, all in a visible
 *    form.
 * 
 *   2.1 If the Application is distributed in a store system (for example,
 *       the Apple "App Store" or "Google Play"), the phrase "powered by heavy"
 *       shall be included in the app description or the copyright text as well as
 *       the in the app itself. The heavy logo will shall be visible in the app
 *       itself as well.
 * 
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO,
 * THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
 * SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER
 * CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
 * OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF
 * THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 * 
 */

#ifndef _HEAVY_CONTEXT_WSTD_SMTHR_HPP_
#define _HEAVY_CONTEXT_WSTD_SMTHR_HPP_

// object includes
#include "HeavyContext.hpp"
#include "HvControlVar.h"
#include "HvSignalLine.h"
#include "HvSignalVar.h"
#include "HvControlIf.h"
#include "HvControlBinop.h"
#include "HvControlCast.h"
#include "HvSignalEnvelope.h"
#include "HvMath.h"

class Heavy_WSTD_SMTHR : public HeavyContext {

 public:
  Heavy_WSTD_SMTHR(double sampleRate, int poolKb=10, int inQueueKb=2, int outQueueKb=0);
  ~Heavy_WSTD_SMTHR();

  const char *getName() override { return "WSTD_SMTHR"; }
  int getNumInputChannels() override { return 2; }
  int getNumOutputChannels() override { return 2; }

  int process(float **inputBuffers, float **outputBuffer, int n) override;
  int processInline(float *inputBuffers, float *outputBuffer, int n) override;
  int processInlineInterleaved(float *inputBuffers, float *outputBuffer, int n) override;

  int getParameterInfo(int index, HvParameterInfo *info) override;
  struct Parameter {
    struct In {
      enum ParameterIn : hv_uint32_t {
        LMTR = 0xCB23DC60, // Lmtr
        MIX = 0x97D46AC6, // Mix
        SMTHR = 0xD88E26F9, // Smthr
      };
    };
  };

 private:
  HvTable *getTableForHash(hv_uint32_t tableHash) override;
  void scheduleMessageForReceiver(hv_uint32_t receiverHash, HvMessage *m) override;

  // static sendMessage functions
  static void cBinop_5h9qYUSw_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cBinop_bkDGCV4t_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cBinop_gItmkrL6_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cBinop_De1M0OiN_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cCast_1Z88tZIl_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cCast_XqItSm4k_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cBinop_pZXSl4Kd_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cMsg_2vKUvRsU_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cBinop_KGAHHi7d_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cBinop_RB3KDQuF_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cBinop_CSz0NY53_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cVar_BZQgxsiy_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cCast_VHb9CdaH_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cVar_W7bCe3G0_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cVar_SPF5lVmi_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cIf_frqCMFFH_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cIf_ZtjvNnTw_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void sEnv_PadoqPa8_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void sEnv_wgdCKB5L_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cBinop_wuQjNnjI_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cMsg_zbfAaBYN_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cBinop_BIQ1mz0b_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cBinop_5UfnO18J_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cCast_RVjzjSe7_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cCast_9ZElyF8s_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cBinop_ynHOWdwP_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cMsg_CEKPcwWS_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cBinop_hRepyENT_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cCast_v2Tj7YCI_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cCast_yYLCzr6s_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cCast_RnbmKGGl_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cVar_Qya1vebo_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cIf_Nzt0xoCa_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cIf_ajBVskLW_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void sEnv_1L7YvHeT_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void sEnv_Diza8yQK_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cBinop_BXamZHmr_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cMsg_dQZHNe3I_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cBinop_GeO11jMQ_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cBinop_ijwj0emK_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cCast_ZpYikLLI_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cCast_ZnnBiUBo_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cBinop_WmB6Ij3p_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cMsg_oZgCXudA_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cBinop_0eq52m5g_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cCast_QRCUMQoJ_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cCast_X7rLMdPB_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cCast_JOTXLP5h_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cVar_2wWO4h0l_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cBinop_GfGchAdC_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cMsg_w8AreF2D_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cMsg_9XmuTWv1_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cSend_lL75Zdy8_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cBinop_iu8hT0Mf_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cMsg_uUsdqzlD_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cMsg_P2TaqbBn_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cMsg_TPmIqdxC_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cReceive_DKkOqRKS_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cReceive_zYZm6197_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cReceive_AM9rqRCd_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cReceive_gvm6HhJ9_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cReceive_JaDmmC1f_sendMessage(HeavyContextInterface *, int, const HvMessage *);

  // objects
  SignalLine sLine_xpnWSdhe;
  SignalEnvelope sEnv_PadoqPa8;
  SignalEnvelope sEnv_wgdCKB5L;
  SignalEnvelope sEnv_1L7YvHeT;
  SignalEnvelope sEnv_Diza8yQK;
  SignalLine sLine_HE0sciXE;
  SignalLine sLine_yNat3HOB;
  ControlBinop cBinop_5h9qYUSw;
  ControlBinop cBinop_bkDGCV4t;
  ControlBinop cBinop_gItmkrL6;
  ControlBinop cBinop_De1M0OiN;
  ControlBinop cBinop_pZXSl4Kd;
  ControlBinop cBinop_KGAHHi7d;
  ControlBinop cBinop_RB3KDQuF;
  ControlBinop cBinop_CSz0NY53;
  ControlVar cVar_BZQgxsiy;
  ControlVar cVar_W7bCe3G0;
  SignalVarf sVarf_wupM3vep;
  ControlVar cVar_SPF5lVmi;
  ControlIf cIf_frqCMFFH;
  ControlIf cIf_ZtjvNnTw;
  ControlBinop cBinop_wuQjNnjI;
  ControlBinop cBinop_BIQ1mz0b;
  ControlBinop cBinop_5UfnO18J;
  ControlBinop cBinop_ynHOWdwP;
  ControlBinop cBinop_hRepyENT;
  SignalVarf sVarf_4iKVg2Ok;
  ControlVar cVar_Qya1vebo;
  ControlIf cIf_Nzt0xoCa;
  ControlIf cIf_ajBVskLW;
  ControlBinop cBinop_BXamZHmr;
  ControlBinop cBinop_GeO11jMQ;
  ControlBinop cBinop_ijwj0emK;
  ControlBinop cBinop_WmB6Ij3p;
  ControlBinop cBinop_0eq52m5g;
  ControlVar cVar_2wWO4h0l;
  ControlBinop cBinop_GfGchAdC;
  SignalVarf sVarf_ip9DHfs6;
  SignalVarf sVarf_zPcpzh0Y;
  SignalVarf sVarf_FEDo29BP;
  SignalVarf sVarf_iXzvIfgr;
  ControlBinop cBinop_iu8hT0Mf;
};

#endif // _HEAVY_CONTEXT_WSTD_SMTHR_HPP_
